#!/bin/sh
#
# Unpacks an update archive and checks its integrity.
#
# @version 1 (13.05.2011) florian.faber@directout.eu
#            - initial version
#
# Exit values:
#   0 - OK
#   1 - Empty parameter
#   2 - File not found
#   3 - Digest verification failed
#  >3 - Errors from install script
#

PUBKEY=/root/.pub.pem
TMPDIR=/tmpdata

if [ $# -ne 2 ]
then
  echo "Usage: $0 <upgrade file> <boundary>"
  exit 1
fi

if [ ! -f $1 ]
then
  echo "File '$1' not found."
  exit 2
fi

/system/multipart_extract $1 file $2 | tar -xo -C $TMPDIR

cd ${TMPDIR}/upgrade

openssl dgst -sha1 -verify $PUBKEY -signature upgrade.tar.bz2.sha1 upgrade.tar.bz2 >/dev/null || exit 3

openssl dgst -sha1 -verify $PUBKEY -signature upgrade.sh.sha1 upgrade.sh >/dev/null || exit 3

rm $1

exit 0
