/**
 * M1k2 gui toolkit
 * @copyright Florian Faber, florian.faber@directout.eu
 * @version 1 (21.01.2012) 
 **/


function create_modal_overlay() {
	if (!document.getElementById('modal_overlay')) {
		var overlay = document.createElement('div');
		overlay.id = 'modal_overlay';
		document.body.appendChild(overlay);
	}
}


function remove_modal_overlay() {
	var ovl = document.getElementById('modal_overlay');
	if (ovl) {
		document.body.removeChild(ovl);
	} else {
		ovl = top.document.getElementById('modal_overlay');
		if (ovl) {
	    top.document.body.removeChild(ovl);
		} else {
			ovl = parent.document.getElementById('modal_overlay');
			if (ovl) {
				parent.document.body.removeChild(ovl);
			}
		}
	}
}


function create_dialog_frame(top_doc, url, id, width, height, zindex, modal) {
	var frame = top_doc.createElement('iframe');

	top_doc.body.appendChild(frame);

	if(frame.contentDocument)
		frame.doc = frame.contentDocument;
	else if(frame.contentWindow)
		frame.doc = frame.contentWindow.document;
	else if(frame.document)
		frame.doc = frame.document;
    
	frame.style.frameborder = 0;
	frame.style.zIndex = zindex;
	frame.id = id;

	frame.setAttribute('src', url);
	frame.className = "dialog_frame";
	frame.style.width = width+"px";
	frame.style.height = height+"px";
	frame.style.marginLeft = (-width/2)+"px";
	frame.style.marginTop = (-height/2)+"px";

	if (modal) {
		create_modal_overlay();
		frame.modal = true;
	}
	return frame;
}


function init_dialog(title, id, width, height, buttons) {
	var win = document.createElement('div');
 
	win.style.width = width+"px";
	win.style.height = height+"px";
	win.className = 'dialog';
	win.id = id;

	var tit = document.createElement('div');
	tit.className = 'dialog_title';
	tit.id = id+"_title";
	tit.innerHTML = title;
	win.appendChild(tit);
    
	var con = document.createElement('div');
	con.className = 'dialog_contents';
	con.id = id+"_contents";
	win.appendChild(con);
    
	if (buttons) {
		var btns = document.createElement('div');
		btns.className = 'dialog_buttons';
		btns.id = id+"_buttons";
		win.appendChild(btns);
	
		con.style.bottom = "38px";
	
	} else {
		con.style.bottom = "10px";
	}
    
	return win;
}
	

function dialog_close(id) {
	var frame = parent.document.getElementById(id);
	if (!frame)
		frame = document.getElementById(id);

	if (frame) {
		if (frame.modal) {
			remove_modal_overlay();
		}
		frame.parentNode.focus();
		frame.parentNode.removeChild(frame);
	}
	/*
		else {
		// framed page
		frame = parent.document.getElementById(id);
		if (!frame)
			frame = document.getElementById(id);
		
		frame.parentNode.focus();
		frame.parentNode.removeChild(frame);
	} 
	/*
		else {
		// single page
		if (frame.modal) {
			remove_modal_overlay();
		}
		top.window.focus();
		top.document.body.removeChild(frame);
	}
	*/
}

function dialog_add(dialog, elem) {
	dialog.children[1].appendChild(elem);
}

function dialog_add_dialogbuttons(dialog, elem) {
	dialog.children[2].appendChild(elem);
}


function create_container(type, alignment) {
	var sep = document.createElement('div');
	sep.className = "gui_container_"+type;
	if (alignment)
		sep.style.textAlign = alignment;

	return sep;
}


function container_add(container, elem) {
	container.appendChild(elem);
}


function create_grid(cols) {
	var tab = document.createElement('table');
	tab.className = "gui_grid";
	tab.cols = cols;
	tab.lastcol = cols;
	tab.lastrow = -1;

	return tab;
}


function grid_add(grid, elem) {
	grid.lastcol++;
	if (grid.lastcol>=grid.cols) {
		grid.lastcol = 0;
		grid.lastrow++;
		var row = document.createElement('tr');
		grid.appendChild(row);
	}
	
	var cell = document.createElement('td');
	cell.style.verticalAlign = 'top';
	var t = typeof elem;
	if (typeof elem=='string') {
		cell.innerHTML = elem;
	} else {
		cell.appendChild(elem);
	}
	
	grid.rows[grid.lastrow].appendChild(cell);
}


function create_separator(titel) {
	var sep = document.createElement('div');
	sep.className = 'gui_separator';
	sep.innerHTML = titel;
	return sep;
}


function create_list(id, width, height, callback_single, callback_double) {
	var list = document.createElement('div');
	list.className = 'gui_list';
	list.id = id;
	list.style.width = width+"px";
	list.style.height = height+"px";

	if (callback_single)
		list.callback_single = callback_single;

	if (callback_double)
		list.callback_double = callback_double;

	return list;
}


function list_add(list, elem, value) {
	elem.value = value;
	elem.className = 'gui_list_elem_nosel';
	elem.onclick = function() { list_select(list, value); if (list.callback_single) list.callback_single(elem); };
	elem.ondblclick = function() { list_select(list, value); if (list.callback_double) list.callback_double(elem); };
	list.appendChild(elem);
}


function list_select(list, value) {
	for (var i=0; i<list.children.length; i++) {
		if (list.children[i].value==value) {
			list.children[i].className = 'gui_list_elem_sel';
		} else {
			list.children[i].className = 'gui_list_elem_nosel';
		}
	}
	list.selvalue = value;
}


function list_get_selected(list) {
	for (var i=0; i<list.children.length; i++) {
		if (list.children[i].className=='gui_list_elem_sel')
			return list.children[i];
	}
	return false;
}


function list_clear(list) {
	list.selvalue = null;
	while (list.hasChildNodes())
		list.removeChild(list.firstChild);
}


function create_radiobuttongroup(id, callback) {
	var rbg = document.createElement('div');
	rbg.id = id;
	rbg.className = 'gui_radiobuttongroup';
	if (callback)
		rbg.callback = callback;

	return rbg;
}

function radiobuttongroup_add(rbg, value, elem) {
	var d = document.createElement('div');
	d.className = 'gui_radiobuttongroup_elem_nosel';
	d.value = value;
	d.innerHTML = elem;
	d.onclick = function() { radiobuttongroup_select(rbg, value); if (rbg.callback) rbg.callback(value); };
	rbg.appendChild(d);
}

function radiobuttongroup_select(rbg, value) {
	for (var i=0; i<rbg.children.length; i++) {
		if (rbg.children[i].value==value) {
			rbg.children[i].className = 'gui_radiobuttongroup_elem_sel';
		} else {
			rbg.children[i].className = 'gui_radiobuttongroup_elem_nosel';
		}
	}
	rbg.selvalue = value;
}


function create_image_button(text, mouseover, image, image_hover, image_press, classname, id, callback) {
	var btn = document.createElement('div');
	btn.className = classname;
	btn.style.backgroundImage = "url("+image+")";
	var txt = document.createElement('div');
	txt.className = classname+"_txt";
	txt.innerHTML = text;
	btn.appendChild(txt);
	btn.callback = callback;
	if (mouseover) {
		txt.title = mouseover;
	  btn.title = mouseover;
	}
	if (id)
		btn.id = id;
	
	btn.onmouseover = function() { btn.style.backgroundImage = "url("+image_hover+")"; };
	btn.onmouseout = function() { btn.style.backgroundImage = "url("+image+")"; };
	btn.onmousedown = function() { btn.style.backgroundImage = "url("+image_press+")"; };
	btn.onmouseup = function() { btn.style.backgroundImage = "url("+image_hover+")"; btn.callback(btn); };
	

	return btn;
}


function real_enable_button(btn) {
	//btn.onmouseover = function() { alert("url(/img/"+btn.className+"_hover.png)"); btn.visited=1; btn.style.backgroundImage = "url(/img/"+btn.className+"_hover.png)"; };
	btn.onmouseover = function() { btn.bla = "url(/img/"+btn.className+"_hover.png)"; btn.style.backgroundImage = "url(/img/"+btn.className+"_hover.png)"; };
	btn.onmouseout = function() { btn.style.backgroundImage = "url(/img/"+btn.className+"_normal.png)"; };
	btn.onmousedown = function() { btn.style.backgroundImage = "url(/img/"+btn.className+"_pressed.png)"; };
	btn.onmouseup = function() { btn.style.backgroundImage = "url(/img/"+btn.className+"_hover.png)"; btn.callback(btn); };
	btn.disabled = false;
	btn.style.opacity = 1;
	btn.style.cursor = 'pointer';
}

function enable_button(id) {
	var btn = document.getElementById(id);
	if (!btn)
		return;
	real_enable_button(btn);
}

function is_enabled(id) {
	var elem = document.getElementById(id);
	return (elem && (elem.disabled==false));
}


function create_button(text, mouseover, classname, id, callback) {
	var btn = document.createElement('div');
	btn.className = classname;
	btn.style.backgroundImage = "url(/img/"+btn.className+"_normal.png)";
	btn.innerHTML = text;
	btn.callback = callback;
	if (mouseover)
		btn.title = mouseover;
	if (id)
		btn.id = id;

	real_enable_button(btn);

	return btn;
}

function disable_button(id) {
	var btn = document.getElementById(id);
	if (!btn)
		return;
	btn.disabled = true;
	btn.style.opacity = 0.5;
	btn.style.cursor = 'progress';
	btn.onmouseover = false;
	btn.onmouseout = false;
	btn.onmousedown = false;
	btn.onmouseup = false;
}
