/**
 * M1k2 channel matrix
 * (c) D.O.Tec, http://www.directout.eu
 * @version 5 (08.11.2018)
 *          - added safety click for jerrick
 * @version 4 (08.02.2017) 
 *          - added raw channel view
 * @version 3 (23.05.2016) Florian Faber, florian.faber@directout.eu
 *          - added port switching
 * @version 2 (07.04.2013) Florian Faber, florian.faber@directout.eu
 *          - switched to WebSocket
 * @version 1 (21.11.2010) Florian Faber, florian.faber@directout.eu
 **/


status_port_mode = [ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ];
config_wck_mul = [ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ];
config_port_mode =  [ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ];
config_lock = Array(1024);
config_xpo = Array(1024);
config_xpa = Array(1024);
config_master_fs = 0;
config_port_input_redundancy = Array(16);
config_port_output_redundancy = Array(16);
config_port_follow = [ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ];

config_master_enable_clock = 0;
config_master_enable_fs = 0;

labels_audio_in = Array(1024);
labels_audio_in_bg = Array(1024);
labels_audio_in_fg = Array(1024);
labels_audio_out = Array(1024);
labels_audio_out_bg = Array(1024);
labels_audio_out_fg = Array(1024);
labels_input_ports = Array(18);
labels_input_ports_bg = Array(20);
labels_input_ports_fg = Array(20);
labels_output_ports = Array(18);
labels_output_ports_bg = Array(20);
labels_output_ports_fg = Array(20);

quick_over = 0;
safety_click = false;

permissions = false;

matrix_width = 20480;
matrix_height = 20480;

// false for any port where all channels are forbidden
permitted_input_ports = Array(16);
permitted_output_ports = Array(16);

// true for any port that should be visible
active_input_ports = Array(16);
active_output_ports = Array(16);

// lut to map ports from displayposition to real ports
port_in = Array(16);
port_out = Array(16);
port_in_rev = Array(16);
port_out_rev = Array(16);

num_input_ports = 0;
num_output_ports = 0;


function get_effective_mul(port) {
    if (config_master_enable_fs)
	return (config_wck_mul[17]==2)?4:(config_wck_mul[17]+1);
    
    if (port<16) {
	if (config_port_output_redundancy[port]!=port) 
	    port = config_port_output_redundancy[port];
	else if (config_port_follow[port]!=port)
	    port = config_port_follow[port];
    } else {
	if (config_port_follow[port]!=port)
	    port = config_port_follow[port];
    }

    if (config_wck_mul[port]==2)
	return 4;
    return config_wck_mul[port]+1;
}


function get_effective_outputmode(port) {
    if (config_port_output_redundancy[port]!=port) 
	port = config_port_output_redundancy[port];
    
    return config_port_mode[port];
}


function esc_master_enable_clock(enable) {
    config_master_enable_clock = enable;
}

function esc_master_enable_fs(enable) {
    config_master_enable_fs = enable;

    for (var i=0; i<16; i++) {
	update_input_port(i);
	update_output_port(i);
    }
}

function esc_port_input_redundancy(port, redundancy) {
    config_port_input_redundancy[port] = redundancy;
}

function esc_port_output_redundancy(port, redundancy) {
    config_port_output_redundancy[port] = redundancy;
}

function esc_port_follow(port, follow) {
    config_port_follow[port] = follow;
}


function update_permission(str) {
    permissions = JSON.parse(str);
    document.getElementById("connecting").style.display = 'none';
    
    for (var i=0; i<16; i++) {
	update_input_port(i);
	update_output_port(i);
    }
    
    for (var i=0; i<1024; i++) {
	var o = document.getElementById("input"+i);
	if (o) {
	    o.className = (permissions.xp_in[i])?"inputch":"inputch disabled";
	}
	o = document.getElementById("output"+i);
	if (o) {
	    o.className = (permissions.xp_out[i])?"outputch":"outputch disabled";
	}
    }

    for (var p=0; p<16; p++) {
	permitted_input_ports[p] = false;
	permitted_output_ports[p] = false;
	for (c=0; (c<64) && (!permitted_input_ports[p] || !permitted_output_ports[p]); c++) {
	    if (permissions.xp_in[p*64+c])
		permitted_input_ports[p] = true;
	    if (permissions.xp_out[p*64+c])
		permitted_output_ports[p] = true;
	}				
    }
    
    update_ports();
}


function permissions_updated(permid) {
    ajax_call('/data/permission', update_permission);
}

function opt_perms_changed(userid, upper, lower) {
    ajax_call('/data/permission', update_permission);
}


/**
 * @param x, y are matrix cell coordinates
 * @returns 1 - ok
 *          2 - port in 56ch mode, channel in 57..64
 *          4 - sample rate differs
 *          8 - speed mode differs
 *         16 - locked
 *         32 - input channel not permitted
 *         64 - output channel not permitted
 **/
function get_xp_state(x, y) {
    var state = 1; // ok
    
    var port_x = port_out[Math.floor(x/64)];
    var ch_x = x%64;
    //var mul_x = config_wck_mul[(config_master_fs)?17:port_x]+1;
    var mul_x = get_effective_mul(port_x);
    //var max_x = config_port_mode[port_x]/mul_x;
    var max_x = get_effective_outputmode(port_x)/mul_x;
    var max_mode_x = 64/mul_x;

    var port_y = port_in[Math.floor(y/64)];
    var ch_y = y%64;
    //var mul_y = config_wck_mul[(config_master_fs)?17:port_y]+1;
    var mul_y = get_effective_mul(port_y);
    var max_y = status_port_mode[port_y]/mul_y;
    var max_mode_y = 64/mul_y;

    if ((ch_x>=max_x) || (ch_y>=max_y))
	state += 2; // OOR (port mode)
    if ((ch_x>=max_mode_x) || (ch_y>=max_mode_y))
	state += 4; // OOR (speedmode)
    if (mul_x != mul_y)
	state += 8; // diff. speed mode  
    if (config_lock[port_x*64+ch_x]==1)
	state += 16; // locked

    if (permissions) {
	if (permissions.xp_in[port_y*64+ch_y]==0)
	    state += 32; // input channel not permitted
	if (permissions.xp_out[port_x*64+ch_x]==0)
	    state += 64; // output channel not permitted
    }

    return state;
}




function channel_accessible_output(och) {
    var port = Math.floor(och/64);
    var ch = och-port*64;
    var max = 64;
    var mul = get_effective_mul(port);

    if (get_effective_outputmode(port)==56)
	max -= 8;
    max = max/mul;

    return (ch<max);
}


function channel_accessible_input(ich) {
    var port = Math.floor(ich/64);
    var ch = ich-port*64;
    var max = 64;
    var mul = get_effective_mul(port);

    if (status_port_mode[port]==56)
	max -= 8;
    max = max/mul;

    return (ch<max);
}

function ch2y(ch) {
    var port = Math.floor(ch/64);
    if (!active_input_ports[port])
	return -1;
    ch = ch%64;

    return port_in_rev[port]*64+ch;
}

function y2ch(y) {
    var port = port_in[Math.floor(y/64)];
    if (!active_input_ports[port])
	return -1;
    var ch = y%64;

    return port*64+ch;	
}

function ch2x(ch) {
    var port = Math.floor(ch/64);
    if (!active_output_ports[port])
	return -1;
    ch = ch%64;

    return port_out_rev[port]*64+ch;
}

function x2ch(x) {
    var port = port_out[Math.floor(x/64)];
    if (!active_output_ports[port])
	return -1;
    var ch = x%64;

    return port*64+ch;	
}


function update_xp_marker(ch, mode) {
    var xp = document.getElementById("xp"+mode+ch);
    var state;

    if (!xp) // Safari fuckup? 2015-04-15
	return;

    var x = ch2x(ch);

    if (mode=='a')
	state = get_xp_state(x, config_xpa[x]);
    else
	//state = get_xp_state(x, config_xpo[x]);
	state = 1;

    xp.state = state;

    if (state & 0x60) {
	xp.src = "/img/matrix_"+mode+"_forbidden.png";
    } else if (state & 0x10) {
	if (state & 0x0E)
	    xp.src = "/img/matrix_"+mode+"_oor_locked.png";
	else
	    xp.src = "/img/matrix_"+mode+"_locked.png";
    } else if (state & 0x0E) {
	xp.src = "/img/matrix_"+mode+"_oor.png";
    }	else if (state==1) {
	xp.src = "/img/matrix_"+mode+".png";
    }
}


function update_input_port(port) {
    var max = 64;
    var mul = get_effective_mul(port);

    if (!permissions)
	return;

    if (status_port_mode[port]==56)
	max -= 8;
    max = max/mul;
    
    for (var i=0; i<64; i++) {
	var input = document.getElementById("input"+(port*64+i));
	if (input) {
	    if ((permissions.xp_in[port*64+i]==1) && (i<max))
		input.className = 'inputch';
	    else
		input.className = 'inputch disabled';
	}
    }

    for (var i=0; i<1024; i++) {
	if ((config_xpa[i]>=port*64) && (config_xpa[i]<(port+1)*64)) 
	    update_xp_marker(i, 'a');

	if ((config_xpo[i]>=port*64) && (config_xpo[i]<(port+1)*64)) 
	    update_xp_marker(i, 'o');
    }    
}


function update_output_port(port) {
    var max = 64;
    var mul = get_effective_mul(port);

    if (!permissions)
	return;

    if (get_effective_outputmode(port)==56)
	max -= 8;
    max = max/mul;
    
    for (var i=0; i<64; i++) {
	var output = document.getElementById("output"+(port*64+i));
	if (output) {
	    if ((permissions.xp_out[port*64+i]==1) && (i<max))
		output.className = 'outputch';
	    else
		output.className = 'outputch disabled';
	    
	    if (config_xpa[port*64+i]>-1)
		update_xp_marker(port*64+i, 'a');
	    
	    if (config_xpo[port*64+i]>-1)
		update_xp_marker(port*64+i, 'o');
	}
    }
}


function ess_port_mode(port, mode) {
    status_port_mode[port] = mode;
    update_input_port(port);
}


function set_name(str) {
    document.title = str+" - Matrix";
}


function esc_xp(m, src, dst) {
    var old = -1;
    var mode;

    if (m==0) {
	mode = 'a';
	old = config_xpa[dst];
	config_xpa[dst] = src;

	if (xpa_pending[dst]!=-2) {
	    matrix_clear_pending(dst, 'a');
	}

    } else if (m==1) {
	mode = 'o';
	old = config_xpo[dst];
	config_xpo[dst] = src;

	if (xpo_pending[dst]!=-2) {
	    matrix_clear_pending(dst, 'o');
	}
    }

    if (active_output_ports[Math.floor(dst/64)]) {														
	// port visible
	
	matrix_clear_xp(old, dst, mode);
	
	if (src>=0)
	    matrix_draw_xp(src, dst, mode);
	
	matrix_update_octet(dst, mode);

	if (show_labels==2) {
	    inherit_labels(dst);
	}
    }
}


function esc_wck_mul(port, mul) {
    config_wck_mul[port] = mul;
    if (port<16) {
	for (var i=0; i<16; i++) {
	    if (config_port_follow[i]==port) {
		update_input_port(i);
		update_output_port(i);
	    }
	    if (config_port_output_redundancy[i]==port) {
		update_output_port(i);
	    }
	}
    } else if ((port==17) && config_master_fs) {
	update_ports();
    }
}

function update_ports() {
    for (var i=0; i<16; i++) {
	update_input_port(i);
	update_output_port(i);
    }  
}


function esc_port_mode(port, mode) {
    config_port_mode[port] = mode;
    update_output_port(port);
}

function esc_al(ch, val) {
    config_lock[ch] = val;

    var port = Math.floor(ch/64);

    if (active_output_ports[port]) {
	var o = document.getElementById("out"+ch+"_l");
	if (o)
	    outputs.removeChild(o);

	if (val==1) {
	    o = document.createElement("img");
	    o.src = "/img/matrix_lock.png";
	    o.id = "out"+ch+"_l";
	    o.className = "xplocked";
	    o.style.zindex = 4;
	    o.style.left = (port_out_rev[port]*1280+(ch%64)*20+5)+"px";
	    o.style.cursor = 'pointer';
	    outputs.appendChild(o);
	    
	    if (config_xpa[ch]>-1)
		update_xp_marker(ch, 'a');
	    if (config_xpo[ch]>-1)
		update_xp_marker(ch, 'o');
	    
	} else {
	    if (config_xpa[ch]>-1)
		update_xp_marker(ch, 'a');
	    if (config_xpo[ch]>-1)
		update_xp_marker(ch, 'o');
	}
    }
}


function esl(type, id, label, fg, bg) {
    switch (type) {
    case 128:
	labels_audio_in[id] = label;
	labels_audio_in_fg[id] = fg;
	labels_audio_in_bg[id] = bg;
	if (show_labels>0) 
	    matrix_setlabel('input', id, label, fg, bg);
	if (show_labels==2) {
	    // find all sources
	    if (matrix_mode=='o') {
		for (var i=0; i<1024; i++) {
		    if (config_xpo[i]==id) {
			inherit_labels(i);
		    }
		}
	    } else if (matrix_mode=='a') {
		for (var i=0; i<1024; i++) {
		    if (config_xpa[i]==id) {
			inherit_labels(i);
		    }
		}
	    }
	}
	break;
    case 129:
	labels_audio_out[id] = label;
	labels_audio_out_fg[id] = fg;
	labels_audio_out_bg[id] = bg;
	if (show_labels==1)
	    matrix_setlabel('output', id, label, fg, bg);
	break;
    case 133:
	if (id==17) {
	    ajax_call('/data/permission', update_permission);
	}   
	break;
    case 134:
	labels_input_ports[id] = label;
	labels_input_ports_fg[id] = fg;
	labels_input_ports_bg[id] = bg;
	if (show_labels>0) 
	    matrix_setport('input', id, label, fg, bg);
	break;
    case 135:
	labels_output_ports[id] = label;
	labels_output_ports_fg[id] = fg;
	labels_output_ports_bg[id] = bg;
	if (show_labels>0)
	    matrix_setport('output', id, label, fg, bg);
	break;
    }
}




function bulk_xp(matrix, ar) {
    var i;
    for (i=0; i<1024; i++)
	esc_xp(matrix, ar[i], i);
}

/*
  function bulk_gain(ar) {
  var i;
  for (i=0; i<1024; i++)
  esc_gain(i, ar[i]);
  }
*/
function bulk_audio_lock(ar) {
    var i;
    for (i=0; i<1024; i++)
	esc_al(i, ar[0][i]);
}

function bulk_label(type, ar) {
    var i;
    
    for (i=0; i<ar.length; i++)
	esl(type, i, ar[i][0], ar[i][1], ar[i][2]);
}


function events_disconnected() {
    document.getElementById("connecting").style.display = 'block';
    connected = 0;
}


function events_connected() {
    connected = 1;
}


channel_routing = null;

matrixmap = null;
mapmoepel = null;
mapmoepelwidth = 0;
mapmoepelheight = 0;
matrix = null;
matrixviewport = null;
matrixmarker = null;
inputs = null;
inputmarker = null;
outputs = null;
outputmarker = null;
marker_x = -1;
marker_y = -1;
marker_size = 1;
show_labels = -1;
lastfound = -1;
matrix_sequence_x = -1;
matrix_sequence_y = -1;
matrix_tool = 0;
// 1=offline, 2=active
matrix_mode = false;
show_all = true;
canvas_online = null;
canvas_online_ctx = null;
canvas_offline = null;
canvas_offline_ctx = null;
xpa_pending = Array(1024);
xpo_pending = Array(1024);

base = 20;

map_drag = false;
matrix_drag = false;
matrix_drag_disabled = false;
matrix_remove = true;

es_config = false;

show_info = 0;


function event_config(ev) {

}


function min(a, b) {
    return (a<b)?a:b;
}


/**
 ** info floaterd 
 **/

function matrix_update_info() {
    var port_x = port_in[Math.floor(matrix_over_x/64)];
    var ch_x = matrix_over_x%64;
    var port_y = port_out[Math.floor(matrix_over_y/64)];
    var ch_y = matrix_over_y%64;

    if (show_labels==3) {
	title = (port_x*64+ch_x+1)+"/"+(port_y*64+ch_y+1);
    } else {
	title = (port_y+1)+"/"+(ch_y+1)+" | "+(port_x+1)+"/"+(ch_x+1);
    }
    document.getElementById("info_coords").innerHTML = title;
    
    var b = document.getElementById("info_in_port");
    b.style.backgroundColor = "#"+labels_input_ports_bg[port_y];
    b.style.color = "#"+labels_input_ports_fg[port_y];
    b.innerHTML = labels_input_ports[port_y];
    
    b = document.getElementById("info_in_ch");
    b.style.backgroundColor = "#"+labels_audio_in_bg[port_y*64+ch_y];
    b.style.color = "#"+labels_audio_in_fg[port_y*64+ch_y];
    b.innerHTML = labels_audio_in[port_y*64+ch_y];

    b = document.getElementById("info_out_port");
    b.style.backgroundColor = "#"+labels_output_ports_bg[port_x];
    b.style.color = "#"+labels_output_ports_fg[port_x];
    b.innerHTML = labels_output_ports[port_x];
    
    b = document.getElementById("info_out_ch");
    b.style.backgroundColor = "#"+labels_audio_out_bg[port_x*64+ch_x];
    b.style.color = "#"+labels_audio_out_fg[port_x*64+ch_x];
    b.innerHTML = labels_audio_out[port_x*64+ch_x];  
    
    //b.innerHTML = get_xp_state(matrix_over_x, matrix_over_y) + " " + document.getElementById("xpa"+matrix_over_x).state;

    var vp_x = Math.floor((matrix_over_x*20+(matrix.offsetLeft))/20);
    var vp_y = Math.floor((matrix_over_y*20+(matrix.offsetTop))/20);
    var floater = document.getElementById("info_floater");
    floater.style.left = (vp_x*20+40)+"px";
    floater.style.top = (vp_y*20+20)+"px";
}


function matrix_show_info() {
    matrix_update_info();
    document.getElementById("info_floater").style.display = 'block';
}

function matrix_hide_info() {
    document.getElementById("info_floater").style.display = 'none';
}

function cb_show_info(res) {
    if (res=="1") {
	show_info = 1;
	matrix_show_info();
    }
}


/**
 ** safety click
 **/

function cb_safety_click(res) {
    safety_click = (res=="1");
}

function matrix_toggle_safety_click() {
    safety_click = !safety_click;
    
    notifications_show("Safe click feature now "+(safety_click?"enabled":"disabled"));

    post_call('/wwwdata?safety_click', safety_click?"1":"0", false);
}


/**
 ** marker handling
 **/

marker_angle = 1;


function matrix_put_marker(x, y) {
    var i, o;
    var states = Array();

    switch (matrix_tool) {
    case 1: // xp
	var valid = true;
	var state=1;
	
	if (matrix_mode=='a') {
	    for (var i=0; i<marker_size; i++) {
		state = get_xp_state(x+i, y+i);
		if (state>1)
		    valid = false;
		states[i] = state;
	    }
	}

	if (valid) {
	    if ('a'==matrix_mode) {
		matrixmarker.style.background = "url(/img/matrixmarker_a.png)";
	    } else {
		matrixmarker.style.background = "url(/img/matrixmarker_o.png)";
		//matrixmarker.style.backgroundColor = "yellow";
	    }
	    inputmarker.style.backgroundColor = 'green';
	    outputmarker.style.backgroundColor = 'green';				       
	    matrix.style.cursor = 'crosshair';
	    
	} else {
	    matrixmarker.style.background = "";
	    if (states[0]>3) {
		matrixmarker.style.backgroundColor = 'red';
		matrix.style.cursor = 'not-allowed';
	    } else {
		matrixmarker.style.backgroundColor = (states[0]>1)?'yellow':'green';
		matrix.style.cursor = 'crosshair';
	    }
	    inputmarker.style.backgroundColor = 'red';
	    outputmarker.style.backgroundColor = 'red';
	    
	}
	
	matrixmarker.style.left = (x*20)+"px";
	matrixmarker.style.top = (y*20)+"px";
	
	break;
	
    case 2: // lock
	var state = get_xp_state(x, y);
	var bg = "#f0ff07";
	if (state<16) {
	    matrix.style.cursor = 'crosshair';
	} else {
	    matrix.style.cursor = 'not-allowed';
	    bg = 'red';
	}
	matrixmarker.style.background = "";
	matrixmarker.style.backgroundColor = bg;
	matrixmarker.style.left = (x*20)+"px";
	matrixmarker.style.top = (-parseInt(matrix.style.top, 10))+"px";
	outputmarker.style.backgroundColor = bg;
	break;

    case 3: // clear
	matrixmarker.style.background = "";
	matrixmarker.style.backgroundColor = "red";
	matrixmarker.style.left = (x*20)+"px";
	matrixmarker.style.top = (y*20)+"px";
	var bg = 'green';
	if ((matrix_mode=='a') && (get_xp_state(x, y)<16)) {
	    matrix.style.cursor = 'crosshair';
	} else {
	    bg = 'red';
	    matrix.style.cursor = 'not-allowed';
	}
	inputmarker.style.backgroundColor = bg;
	outputmarker.style.backgroundColor = bg;				
	break;
    }

    marker_x = x;
    marker_y = y;
    
    if ((marker_size>1) && (matrix_tool==1))  {
	for (i=1; i<marker_size; i++) {
	    o = document.getElementById('ghost'+i);
	    o.style.backgroundColor = (states[i]>3)?'red':(states[i]>1)?'yellow':'white';
	    o.style.left = ((marker_x+i)*20)+"px";
	    if (marker_angle==0) { // horizontal
		o.style.top = (marker_y*20)+"px";
	    } else if (marker_angle==1) { // diagonal SE
		o.style.top = ((marker_y+i)*20)+"px";
	    } else if (marker_angle==2) { // diagonal NE
		o.style.top = ((marker_y-i)*20)+"px";
	    }
	}
    }
}


function matrix_create_ghost(id, x, y) {
    var o;

    o = document.createElement("div");
    o.id = 'ghost'+id;
    o.className = 'matrixghost';
    o.style.top = (y*20)+"px";
    o.style.left = (x*20)+"px";
    matrix.appendChild(o);
}


function matrix_remove_ghost(id) {
    var o = document.getElementById('ghost'+id);
    if (o) {
	matrix.removeChild(o);
    }
}


function matrix_set_marker_size(s) {
    var i;
    
    if (s<1) return;

    switch (matrix_tool) {
    case 1: // set
	if (s>marker_size) {
	    for (i=marker_size; i<s; i++) {
		if (marker_angle==0) { // horizontal
		    matrix_create_ghost(i, marker_x+i, marker_y);
		} else if (marker_angle==1) { // diagonal SE
		    matrix_create_ghost(i, marker_x+i, marker_y+i);
		} else if (marker_angle==2) { // diagonal NE
		    matrix_create_ghost(i, marker_x+i, marker_y-i);
		}
	    }
	} else if(s<marker_size) {
	    for (i=marker_size; i>=s; i--) {
		matrix_remove_ghost(i);
	    }
	}
	matrixmarker.style.width = "12px";
	matrixmarker.style.height = "12px";
	matrixmarker.style.margin = "4px";
	inputmarker.style.display = 'block';
	break;

    case 2: // lock
	matrixmarker.style.width = (20*s)+"px";
	matrixmarker.style.height = matrixviewport.clientHeight+"px";
	matrixmarker.style.margin = 0;
	inputmarker.style.display = 'none';
	break;

    case 3: // wipe
	matrixmarker.style.width = (20*s-8)+"px";
	matrixmarker.style.height = (20*s-8)+"px";
	matrixmarker.style.margin = "4px";
	inputmarker.style.display = 'block';
	break;
    }

    marker_size = s;
    matrix_put_marker(marker_x, marker_y);

    if (marker_angle==0) {
	inputmarker.style.top = (marker_y*20+parseInt(inputs.style.top, 10))+"px";
	inputmarker.style.height = "20px";
    } else if (marker_angle==1) {
	inputmarker.style.top = (marker_y*20+parseInt(inputs.style.top, 10))+"px";
	inputmarker.style.height = (20*s)+"px";
    } else if (marker_angle==2) {
	inputmarker.style.top = (marker_y*20-(marker_size-1)*20+parseInt(inputs.style.top, 10))+"px";
	inputmarker.style.height = (20*s)+"px";
    }
    outputmarker.style.width = (20*s)+"px";
}


function set_marker_angle(new_angle) {
    marker_angle = new_angle;
    matrix_set_marker_size(marker_size);
}


/**
 ** Show other
 **/

function matrix_set_show_other(val) {
    show_all = val;
    matrix_update_xps();
    matrix_update_canvas();
    post_call('/wwwdata?matrix_show_other', show_all?"1":"0", false);
}

function cb_show_other(res) {
    matrix_set_show_other(res=="1");
}

/**
 ** label handling
 **/


function inherit_labels(ch) {
    if (matrix_mode=='o') {
	if (config_xpo[ch]>=0) {
	    matrix_setlabel('output', ch, labels_audio_in[config_xpo[ch]], labels_audio_in_fg[config_xpo[ch]], labels_audio_in_bg[config_xpo[ch]]);
	} else {
	    matrix_setlabel('output', ch, "N/C", "000000", "FFFFFF");
	}
    } else {
	if (config_xpa[ch]>=0) {
	    matrix_setlabel('output', ch, labels_audio_in[config_xpa[ch]], labels_audio_in_fg[config_xpa[ch]], labels_audio_in_bg[config_xpa[ch]]);
	} else {
	    matrix_setlabel('output', ch, "N/C", "000000", "FFFFFF");
	}

    }
}

function redraw_labels() {
    if (show_labels==3) {
	// RAW CHANNELS
	for (i=0; i<num_input_ports; i++) {
	    if (i%2==0) {
		bg = "404040";
	    } else {
		bg = "505050";
	    }
	    matrix_setport('input', port_in[i], "Input port "+(port_in[i]+1), 'ffffff', bg);
	    for (c=0; c<64; c++) {
		if (c%16<8) {
		    bg = "606060";
		} else {
		    bg = "707070";
		}
		matrix_setlabel('input', port_in[i]*64+c, "Input "+((port_in[i]*64)+(c+1)), 'ffffff', bg);								
	    }
	}
	
	for (i=0; i<num_output_ports; i++) {
	    if (i%2==0) {
		bg = "404040";
	    } else {
		bg = "505050";
	    }
	    matrix_setport('output', port_out[i], "Output port "+(port_out[i]+1), 'ffffff', bg);
	    for (c=0; c<64; c++) {
		if (c%16<8) {
		    bg = "606060";
		} else {
		    bg = "707070";
		}
		matrix_setlabel('output', port_out[i]*64+c, "Output "+((port_out[i]*64)+(c+1)), 'ffffff', bg);
		
	    }
	}
    } else if (show_labels==2) {
	// INHERIT
	for (i=0; i<num_input_ports; i++) {
	    matrix_setport('input', port_in[i], labels_input_ports[port_in[i]], labels_input_ports_fg[port_in[i]], labels_input_ports_bg[port_in[i]]);
	    for (c=0; c<64; c++) {
		matrix_setlabel('input', port_in[i]*64+c, labels_audio_in[port_in[i]*64+c], labels_audio_in_fg[port_in[i]*64+c], labels_audio_in_bg[port_in[i]*64+c]);
	    }								
	}
	for (i=0; i<num_output_ports; i++) {
	    matrix_setport('output', port_out[i], labels_output_ports[port_out[i]], labels_output_ports_fg[port_out[i]], labels_output_ports_bg[port_out[i]]);
	    for (c=0; c<64; c++) {
		inherit_labels(port_out[i]*64+c);
	    }
	}

    } else if (show_labels==1) {
	// CUSTOM LABELS
	for (i=0; i<num_input_ports; i++) {
	    matrix_setport('input', port_in[i], labels_input_ports[port_in[i]], labels_input_ports_fg[port_in[i]], labels_input_ports_bg[port_in[i]]);
	    for (c=0; c<64; c++) {
		matrix_setlabel('input', port_in[i]*64+c, labels_audio_in[port_in[i]*64+c], labels_audio_in_fg[port_in[i]*64+c], labels_audio_in_bg[port_in[i]*64+c]);
	    }	
	}
	for (i=0; i<num_output_ports; i++) {
	    matrix_setport('output', port_out[i], labels_output_ports[port_out[i]], labels_output_ports_fg[port_out[i]], labels_output_ports_bg[port_out[i]]);
	    for (c=0; c<64; c++) {
		matrix_setlabel('output', port_out[i]*64+c, labels_audio_out[port_out[i]*64+c], labels_audio_out_fg[port_out[i]*64+c], labels_audio_out_bg[port_out[i]*64+c]);
	    }
	}

    } else if (show_labels==0) {
	// SYSTEM LABELS
	for (i=0; i<num_input_ports; i++) {
	    if (i%2==0) {
		bg = "404040";
	    } else {
		bg = "505050";
	    }
	    matrix_setport('input', port_in[i], "Input port "+(port_in[i]+1), 'ffffff', bg);
	    for (c=0; c<64; c++) {
		if (c%16<8) {
		    bg = "606060";
		} else {
		    bg = "707070";
		}
		matrix_setlabel('input', port_in[i]*64+c, "Input "+(port_in[i]+1)+'/'+(c+1), 'ffffff', bg);								
	    }
	}
	
	for (i=0; i<num_output_ports; i++) {
	    if (i%2==0) {
		bg = "404040";
	    } else {
		bg = "505050";
	    }
	    matrix_setport('output', port_out[i], "Output port "+(port_out[i]+1), 'ffffff', bg);
	    for (c=0; c<64; c++) {
		if (c%16<8) {
		    bg = "606060";
		} else {
		    bg = "707070";
		}
		matrix_setlabel('output', port_out[i]*64+c, "Output "+(port_out[i]+1)+'/'+(c+1), 'ffffff', bg);
		
	    }
	}
    }
}


function matrix_toggle_labels() {
    var i, o, bg;

    show_labels++;
    if (show_labels>3)
	show_labels = 0;

    notifications_show("Labels now show "+((show_labels==0)?"system labels":(show_labels==1)?"custom labels":(show_labels==2)?"inherited labels":"raw numbers"));

    redraw_labels();
    
    post_call('/wwwdata?show_labels', show_labels.toString(), false);
}


function cb_show_labels(res) {
    if (res<0)
	res = 0
    if (res>3)
	res = 3;

    show_labels = res;
    redraw_labels();
}


function matrixmoepel_scroll(x, y) {
    //mapmoepel.style.left = min(x/(20480/160), 160-mapmoepelwidth-2)+"px";
    //mapmoepel.style.top = min(y/(20480/160), 160-mapmoepelheight-2)+"px";
    mapmoepel.style.left = min((x/(num_output_ports*1280))*num_output_ports*10, num_output_ports*10-mapmoepelwidth-2)+"px";
    mapmoepel.style.top = min((y/(num_input_ports*1280))*num_input_ports*10, num_input_ports*10-mapmoepelheight-2)+"px";
}


function matrix_scroll(x, y) {
    if (x<0) x=0;
    if (y<0) y=0;
    if (x > matrix_width-window.innerWidth+160) x=matrix_width-window.innerWidth+160;
    if (y > matrix_height-window.innerHeight+160) y=matrix_height-window.innerHeight+160;
    
    x = Math.floor(x);
    y = Math.floor(y);
    
    matrixmoepel_scroll(x, y);
    
    matrix.style.left = -x+"px";
    matrix.style.top = -y+"px";
    
    inputs.style.top = -y+"px";
    outputs.style.left = -x+"px";
    
    lastfound = -1;
}


function matrix_quadrant(dir) {
    var current, x, y;
    x = -parseInt(matrix.style.left, 10);
    y = -parseInt(matrix.style.top, 10);

    /**
     * 0 1
     * 2 3
     **/
    if (x<10240) {
	if (y<10240) {
	    current = 0;
	} else {
	    current = 2;
	}
    } else {
	if (y<10240) {
	    current = 1;
	} else {
	    current = 3;
	}
    }

    current += dir;
    if (current<0) current = 3;
    if (current>3) current = 0;

    switch (current) {
    case 0: matrix_scroll(0, 0); break;
    case 1: matrix_scroll(10240, 0); break;
    case 2: matrix_scroll(0, 10240); break;
    case 3: matrix_scroll(10240, 10240); break;
    }
}


function matrix_outputjump(ch, mode) {
    var x, y;

    x = -parseInt(matrix.style.left, 10);
    if (mode=='a') {
	y = config_xpa[ch]*20-matrixviewport.offsetHeight/2;
	matrix_put_marker(ch, config_xpa[ch]);
    }	else {
	y = config_xpo[ch]*20-matrixviewport.offsetHeight/2;
	matrix_put_marker(ch, config_xpo[ch]);
    }

    if (y<0) y=0;

    matrix_scroll(x, y);
}


function matrix_inputjump(chx, mode) {
    var start, i, x, y;

    var channel_routing;
    if (mode=='a')
	channel_routing = config_xpa;
    else
	channel_routing = config_xpo;

    if (lastfound==-1) {
	start = Math.floor((matrixviewport.offsetWidth/2 + (-parseInt(matrix.style.left, 10)))/20);
    } else {
	start = lastfound;
    }
    i = start+1;
    while (i != start) {
	if (channel_routing[i]==chx) {
	    matrix_put_marker(i, channel_routing[i]);
	    x = i*20-matrixviewport.offsetWidth/2;
	    y = -parseInt(matrix.style.top, 10);

	    if (x<0) x = 0;
	    matrix_scroll(x, y);
	    lastfound = i;
	    return;
	}
	
	i++;
	if (i>1023) i = 0;
    }
}


/*
 * reflect changes in port geomatry in matrix map
 */ 
function matrixmap_redraw() {
    if ((num_input_ports==16) && (num_output_ports==16)) {
	matrixmap.style.background = "url(/img/matrixmap_bg.png)";
	matrixmap.style.width = "160px";
	matrixmap.style.height = "160px";
	matrixmap.style.left = "0";
	matrixmap.style.bottom = "0";
	canvas_online.style.left = "0";
	canvas_online.style.bottom = "0";
	canvas_offline.style.left = "0";
	canvas_offline.style.bottom = "0";
	
    } else {
	matrixmap.style.background = "url(/img/matrixmap_bg_grey.png)";
	matrixmap.style.width = (num_output_ports*10)+"px";
	matrixmap.style.height = (num_input_ports*10)+"px";
	matrixmap.style.left = (16-num_output_ports)*5+"px";
	matrixmap.style.bottom = (16-num_input_ports)*5+"px";
	canvas_online.style.left = (16-num_output_ports)*5+"px";
	canvas_online.style.bottom = (-(16-num_input_ports)*5)+"px";
	canvas_online_ctx.clearRect(0, 0, 160, 160);
	canvas_offline.style.left = (16-num_output_ports)*5+"px";
	canvas_offline.style.bottom = (-(16-num_input_ports)*5)+"px";
	canvas_offline_ctx.clearRect(0, 0, 160, 160);
    }
}

function matrixmap_scroll(x, y) {
    y -= matrixmap.offsetTop;
    x -= matrixmap.offsetLeft;
    //matrix_scroll(x*(20480/160), y*(20480/160));
    //matrix_scroll((Math.floor(x-(16-num_output_ports)*5)/10)*1280, (Math.floor(y/10)*1280);
    matrix_scroll(Math.floor(x/10)*1280, Math.floor(y/10)*1280);
}

function matrixmap_moepelresize() {
    mapmoepelwidth = (160/(20480/window.innerWidth));
    mapmoepelheight = (160/(20480/window.innerHeight));
    mapmoepel.style.width = mapmoepelwidth+"px";
    mapmoepel.style.height = mapmoepelheight+"px";
}  


function matrix_enabledrag() {
    matrix_drag_disabled = false;
}


function matrix_disabledrag() {
    matrix_drag = false;
    matrix_drag_disabled = true;
    window.setTimeout('matrix_enabledrag()', 200);
}


function matrix_enableremove() {
    matrix_remove = true;
}


function matrix_disableremove() {
    matrix_remove = false;
    window.setTimeout('matrix_enableremove()', 500);
}


function matrixmap_mousedown(e) {
    map_drag = true;
    matrixmap_scroll(e.clientX, e.clientY);
    e.preventDefault();
    return false;
}

function matrixmap_mouseup(e) {
    map_drag = false;
    e.preventDefault();
    return false;
}

function matrixmap_mousemove(e) {
    //if (map_drag) {
    if (e.button==1)
	matrixmap_scroll(e.clientX, e.clientY);
    //} 
    
    e.preventDefault();
    return false;
}

function matrixmap_mouseout(e) {
    if ((e.relatedTarget != mapmoepel) && (e.relatedTarget != matrixmap)) {
	map_drag = false;
    }
}


/*
 * @params x,y are matrix cell coordinates
 */
function matrix_toggle_xp(x, y) {
    var dst = port_out[Math.floor(x/64)]*64+x%64;
    var src = port_in[Math.floor(y/64)]*64+y%64;
    
    if (matrix_tool==2) {
	matrix_action(x, y, 1);

    } else if (!((matrix_mode=='a') && (config_lock[dst]>0))) {
	if (channel_routing[dst] == src) {
	    if (matrix_tool==1) {
		if (matrix_remove) {
		    // remove xp
		    matrix_remove_xp(dst, matrix_mode);
		    matrix_disabledrag();
		}
	    } else {
		matrix_action(x, y, 1);
	    }
	} else {
	    matrix_action(x, y, 1);
	}
    }
}



function matrix_lock_channel(ch) {
    if (config_lock[ch]>0) return;
    send_cmd("lock?"+ch+",1");
} 


function matrix_unlock_channel(ch) {
    if (config_lock[ch]==0) 
	return;
    send_cmd("lock?"+ch+",0");
}


function matrix_action(x, y, size) {
    var i;
    var states = Array();
    var src = y2ch(y);
    var dst = x2ch(x);
    
    switch (matrix_tool) {
    case 1:
	//var all_set = (config_lock[dst]==1) || ((config_lock[dst]==0) && (((matrix_mode=='a') && (config_xpa[dst]==src)) || ((matrix_mode=='o') && (config_xpo[dst]==src))));
	//var all_set = (((matrix_mode=='a') && (config_xpa[dst]==src)) || ((matrix_mode=='o') && (config_xpo[dst]==src)));
	var all_set = true;

	if (marker_angle==0) { // horizontal
	    for (i=0; i<size; i++) {
		if (matrix_mode=='a') {
		    states[i] = get_xp_state(ch2x(dst+i), ch2y(src));
		    if (states[i]<16) {
			if (config_xpa[dst+i]!=src)
			    all_set = false;
		    }
		} else {
		    states[i] = 1;
		    if (config_xpo[dst+i]!=src)
			all_set = false;
		}
	    }
	} else if (marker_angle==1) { // diagonal SE
	    for (i=0; i<size; i++) {
		if (matrix_mode=='a') {
		    states[i] = get_xp_state(ch2x(dst+i), ch2y(src+i));
		    if (states[i]<16) {
			if (config_xpa[dst+i]!=src+i)
			    all_set = false;
		    }
		} else {
		    states[i] = 1;
		    if (config_xpo[dst+i]!=src+i)
			all_set = false;
		}
	    }

	} else if (marker_angle==2) { // diagonal NE
	    for (i=0; i<size; i++) {
		if (matrix_mode=='a') {
		    states[i] = get_xp_state(ch2x(dst+i), ch2y(src-i));
		    if (states[i]<16) {
			if (config_xpa[dst+i]!=src-i)
			    all_set = false;
		    }
		} else {
		    states[i] = 1;
		    if (config_xpo[dst+i]!=src-i)
			all_set = false;
		}
	    }
	}
	
	for (i=0; i<size; i++) {
	    if (states[i]<4) { 								
		if (!all_set) {
		    if (marker_angle==0) { // horizontal
			if (((matrix_mode=='a') && (config_xpa[x2ch(x+i)]!=src)) || ((matrix_mode=='o') && (config_xpo[x2ch(x+i)]!=src)))
			    matrix_set_xp(src, x2ch(x+i), matrix_mode);
		    } else if (marker_angle==1) { // diagonal SE
			if (((matrix_mode=='a') && (config_xpa[x2ch(x+i)]!=src+i)) || ((matrix_mode=='o') && (config_xpo[x2ch(x+i)]!=y2ch(y+i))))
			    matrix_set_xp(y2ch(y+i), x2ch(x+i), matrix_mode);
		    } else if (marker_angle==2) { // diagonal NE
			if (((matrix_mode=='a') && (config_xpa[x2ch(x+i)]!=src-i)) || ((matrix_mode=='o') && (config_xpo[x2ch(x+i)]!=y2ch(y-i))))
			    matrix_set_xp(y2ch(y-i), x2ch(x+i), matrix_mode);
		    }
		} else {
		    if (marker_angle==0) { // horizontal
			if (((matrix_mode=='a') && (config_xpa[x2ch(x+i)]==src)) || ((matrix_mode=='o') && (config_xpo[x2ch(x+i)]==src)))
			    matrix_remove_xp(x2ch(x+i), matrix_mode);
		    } else if (marker_angle==1) { // diagonal SE
			if (((matrix_mode=='a') && (config_xpa[x2ch(x+i)]==src+i)) || ((matrix_mode=='o') && (config_xpo[x2ch(x+i)]==y2ch(y+i))))
			    matrix_remove_xp(dst+i, matrix_mode);
		    } else if (marker_angle==2) { // diagonal NE
			if (((matrix_mode=='a') && (config_xpa[x2ch(x+i)]==src-i)) || ((matrix_mode=='o') && (config_xpo[x2ch(x+i)]==y2ch(y-i))))
			    matrix_remove_xp(x2ch(x+i), matrix_mode);
		    }
		}
		matrix_disableremove();
	    }
	}
	break;

    case 2:
	var all_locked = true;

	for (i=dst; i<dst+size; i++) {
	    if (config_lock[i]==0) {
		all_locked = false;
		i = dst+size;
	    }
	}

	for (i=dst; i<dst+size; i++) {
	    if (!all_locked) {
		matrix_lock_channel(i);
	    } else {
		matrix_unlock_channel(i);
	    }
	}
	break;

    case 3:
	matrix_delete();
	break;
    }

    matrix_disabledrag();
}


function matrix_delete() {
    var i;
    for (i=marker_x; i<marker_x+marker_size; i++) {
	ch = x2ch(i);
	if ((channel_routing[ch] >= y2ch(marker_y)) && (channel_routing[ch] < y2ch(marker_y)+marker_size)) {
	    if (matrix_mode=='o' || (matrix_mode=='a' && config_lock[ch]==0)) {
		matrix_remove_xp(ch, matrix_mode);
	    }
	}
    }
}


function matrix_clear() {
    send_cmd("clear_matrix?"+((matrix_mode=='a')?"0":"1"));
}


matrix_click_x = -1;
matrix_click_y = -1;
matrix_scrollstart_x = -1;
matrix_scrollstart_y = -1;
matrix_moved = false;


function matrix_mousedown(e) {
    if (e.button==0) {
	if (!matrix_drag_disabled) 
	    matrix_drag= true;
	matrix_moved = false;
	matrix_click_x = e.clientX;
	matrix_click_y = e.clientY;
	matrix_scrollstart_x = parseInt(matrix.style.left, 10);
	matrix_scrollstart_y = parseInt(matrix.style.top, 10);
    }
    e.preventDefault();
    return false;
}

function matrix_mouseup(e) {
    var x, y, i;

    if (e.button==0) {
	matrix_drag = false;
	//matrix.style.cursor = 'crosshair';

	if (safety_click && !e.ctrlKey) {
	    notifications_show("Safe click enabled. Please hold down Ctrl key while clicking!");
	} else {
	    if (!matrix_moved) {
		x = Math.floor((e.clientX-161 + (-parseInt(matrix.style.left, 10)))/20);
		y = Math.floor((e.clientY + (-parseInt(matrix.style.top, 10)))/20);
		
		if (matrix_sequence_x>-1) {
		    matrix_action(matrix_sequence_x, matrix_sequence_y, x-matrix_sequence_x+1);
		    matrix_sequence_x = -1;
		    inputmarker.style.height = "20px";
		    outputmarker.style.width = "20px";
		    
		} else {
		    if (e.shiftKey) {
			matrix_sequence_x = x;
			matrix_sequence_y = y;
			matrix.style.cursor = 'se-resize';
			
		    } else {
			if (marker_size==1) {
			    matrix_toggle_xp(x, y);
			} else {
			    matrix_action(x, y, marker_size);
			}
		    }
		}
	    }
	}
    }

    e.preventDefault();
    return false;
}

function matrix_mouseout(e) {
    if ((e.relatedTarget != matrixmarker) && (e.relatedTarget != matrix)) {
	matrix_drag = false;
	matrix.style.cursor = 'crosshair';
	inputmarker.style.display = 'none';
	outputmarker.style.display = 'none';
	if (show_info)
	    matrix_hide_info();
    }
}

function matrix_mouseenter(e) {
    matrixmarker.style.display = 'block';
    if (matrix_tool != 2) 
	inputmarker.style.display = 'block';
    outputmarker.style.display = 'block';
    if (show_info)
	matrix_show_info();
}



matrix_over_x = -1;
matrix_over_y = -1;

function matrix_mousemove(e) {
    var accel = 1.0;
    var over_x, over_y;
    var off_x, off_y;

    if (matrix_drag) {
	off_x = e.clientX-matrix_click_x;
	off_y = e.clientY-matrix_click_y;
	if (matrix_moved || (off_x>5) || (off_x<-5) || (off_y>5) || (off_y<-5)) {
	    matrix_moved = true;
	    matrix.style.cursor = 'pointer';
	    if (e.shiftKey) {
		accel = 8.0;
	    } else if (e.ctrlKey || e.metaKey) {
		accel = 64.0;
	    }
	    matrix_scroll(-(off_x*accel+matrix_scrollstart_x), -(off_y*accel+matrix_scrollstart_y));
	} 
    }

    over_x = Math.floor((e.clientX-161 + (-parseInt(matrix.style.left, 10)))/20);
    over_y = Math.floor((e.clientY + (-parseInt(matrix.style.top, 10)))/20);
    
    if ((over_x != matrix_over_x) || (over_y != matrix_over_y)) {
	if (matrix_sequence_x>-1) {
	    //inputmarker.style.height = ((min(over_y-matrix_sequence_y, over_x-matrix_sequence_x)+1)*20)+"px";
	    //outputmarker.style.width = ((min(over_y-matrix_sequence_y, over_x-matrix_seque8nce_x)+1)*20)+"px";
	    //inputmarker.style.heig ht = ((over_x-matrix_sequence_x+1)*20)+"px";
	    //outputmarker.style.width = ((over_x-matrix_sequence_x+1)*20)+"px";
	    matrix_set_marker_size(over_x-matrix_sequence_x+1);
	} else {
	    
	    matrix_put_marker(over_x, over_y);
	}
	
	if (over_y!=matrix_over_y) {
	    if (marker_angle==2) {
		inputmarker.style.top = (over_y*20-(marker_size-1)*20+parseInt(inputs.style.top, 10))+"px";
	    } else {
		inputmarker.style.top = (over_y*20+parseInt(inputs.style.top, 10))+"px";
	    }
	}
	
	if (over_x!=matrix_over_x) {
	    outputmarker.style.left = (over_x*20+parseInt(outputs.style.left, 10))+"px";
	}

	matrix_over_x = over_x;
	matrix_over_y = over_y;

	if (show_info)
	    matrix_update_info();
    }

    e.preventDefault();
    return false;
}


function matrix_snapshot(shift) {
    if (shift) {
	/*
	  if (matrix_mode=='a')
	  window.open('/data/matrix_online');
	  else
	  window.open('/data/matrix_offline');
	*/
    } else {
	snapshot();
    }
}


function matrix_recall_trigger(shift) {
    if (shift) {
	var inp = frames[0].document.getElementById('snapshot_name');
	inp.click();
    } else {
	restore();
    }
}


function matrix_recall(form) {
    form.action += "?"+((matrix_mode=='a')?"0":"1");
    form.submit();
    top.window.focus();
}


function matrix_keyscroll(x, y, shift, ctrl) {
    var base = 1, vx, vy;

    vx = Math.floor(-parseInt(matrix.style.left, 10)/20);
    vy = Math.floor(-parseInt(matrix.style.top, 10)/20);

    if (ctrl) base = 64; else if (shift) base = 8;

    if (x != 0) {
	vx = Math.floor(vx/base)*base+x*base;
    }
    if (y != 0) {
	vy = Math.floor(vy/base)*base+y*base;
    }

    matrix_scroll(vx*20, vy*20);

    matrix_over_x += vx;
    matrix_over_y += vy;
}


function matrix_mousewheel(e) {
    if (e.wheelDelta) {
	if (e.wheelDeltaX==120) {
	    matrix_keyscroll(-1, 0, true, e.shiftKey);
	} else if (e.wheelDeltaX==-120) {
	    matrix_keyscroll(1, 0, true, e.shiftKey);
	}
	if (e.wheelDeltaY==-120) {
	    matrix_keyscroll(0, 1, true, e.shiftKey);
	} else if (e.wheelDeltaY==120) {
	    matrix_keyscroll(0, -1, true, e.shiftKey);
	}
    } else if (e.detail) {
	if (e.detail==1) {
	    matrix_keyscroll(1, 0, true, false);
	} else if (e.detail==-1) {
	    matrix_keyscroll(-1, 0, true, false);
	} else if (e.detail==-3) {
	    matrix_keyscroll(0, -1, true, false);
	} else if (e.detail==3) {
	    matrix_keyscroll(0, 1, true, false);
	}
    }
}

var input_active=false;

function matrix_add_src_input(channel) {
    var inp = document.createElement("input");
    inp.type = 'text';
    inp.value = labels_audio_in[channel];
    inp.className = 'srclabelinput';
    inp.labeltype = 'src';
    inp.labelid = channel;
    inp.maxLength = 16;
    inp.onchange = matrix_labelchanged;
    inp.onblur = matrix_closeinput;
    inp.id = 'srclabelinput_'+inp.labelid;
    
    var c = document.getElementById('input'+channel);
    c.appendChild(inp);
    
    inp.focus();
    inp.select();
    input_active = true;
}


function matrix_add_dst_input(channel) {
    var inp = document.createElement("input");
    inp.type = 'text';
    inp.value = labels_audio_out[channel];
    inp.className = 'dstlabelinput';
    inp.labeltype = 'dst';
    inp.labelid = channel;
    inp.onchange = matrix_labelchanged;
    inp.onblur = matrix_closeinput;
    inp.id = 'dstlabelinput_'+inp.labelid;
    
    var c = document.getElementById('output'+channel);
    c.appendChild(inp);
    
    inp.focus();
    inp.select();
    input_active = true;
}


function matrix_closeinput(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    var c;
    
    if (srcElement.labeltype=='src')
	c = document.getElementById('input'+srcElement.labelid);
    else
	c = document.getElementById('output'+srcElement.labelid);

    //c.removeChild(e.srcElement);
    c.removeChild(c.children[0]);

    input_active = false;
}


function matrix_labelchanged(e) {
    var fg, bg, type, srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    srcElement.onchange = null;
    srcElement.onblur = null;
    
    if (srcElement.labeltype=='src') {
	type = 128;
	fg = labels_audio_in_fg[srcElement.labelid];
	bg = labels_audio_in_bg[srcElement.labelid];
    } else {
	type = 129;
	fg = labels_audio_out_fg[srcElement.labelid];
	bg = labels_audio_out_bg[srcElement.labelid];
    }

    post_cmd("label?"+type+","+srcElement.labelid+","+fg+","+bg, srcElement.value);

    matrix_closeinput(e);
    return false;
}



function matrix_input_click(e) {
    var srcElement;
    
    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    if (typeof(srcElement.channel)==="undefined")
	return;
    
    if (show_labels)
	matrix_add_src_input(srcElement.channel);
    else
	alert("System labels cannot be edited. Please switch to user labels (press 'd') first!");
}


function matrix_output_click(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }
    
    if (srcElement.channel===undefined)
	return;
    
    if (show_labels)
	matrix_add_dst_input(srcElement.channel);
}


function matrix_addinput(id, pos) {
    var o = document.createElement("div");
    o.className = 'inputch';
    o.id = 'input'+id;
    o.channel = id;
    o.style.top = pos+"px";
    o.ondblclick = matrix_input_click;
    inputs.appendChild(o);
}

function matrix_addoutput(id, pos) {
    var o = document.createElement("div");
    o.className = 'outputch';
    o.id = 'output'+id;
    o.channel = id;
    o.style.left = (pos+10)+"px";
    o.style.top = 140+"px";
    o.ondblclick = matrix_output_click;
    outputs.appendChild(o);
}


function matrix_setlabel(type, id, name, fg, bg) {
    if (typeof(fg)=='undefined')
	return;
    var o = document.getElementById(type+id);
    if (o) {
	o.innerHTML = name;
	o.style.color = '#'+fg;
	o.style.backgroundColor = '#'+bg;
    }
}


function matrix_addinputport(id, pos, name, bgcolor) {
    var o = document.createElement("div");
    o.className = 'inputport';
    o.id = 'inputport'+id;
    o.style.top = (pos+1280-15)+"px";
    o.style.left = "15px";
    inputs.appendChild(o);
}


function matrix_addoutputport(id, pos, name, bgcolor) {
    var o = document.createElement("div");
    o.className = 'outputport';
    o.id = 'outputport'+id;
    o.innerHTML = name;
    o.style.left = (pos+1)+"px";
    if (bgcolor) o.style.backgroundColor = '#'+bgcolor;
    outputs.appendChild(o);
}


function matrix_setport(type, id, name, fg, bg) {
    if (typeof(fg)=='undefined')
	return;
    var o = document.getElementById(type+'port'+id);
    if (o) {
	o.innerHTML = "<div style='float: left; margin-left: 20px;'>"+name+"</div>"+name+"<div style='float: right; margin-right: 20px'>"+name+"</div>";
	o.style.color = '#'+fg;
	o.style.backgroundColor = '#'+bg;
	o.style.borderRight = '1px solid #'+fg;
    }
}


function matrix_toggle_help() {
    var h = document.getElementById('help');
    if (h.style.display!='block') {
	var height = window.innerHeight-200;
	if (height>500)
	    height = 500;
	h.style.height = height+'px';
	h.style.marginTop = -(height/2)+'px';
	h.style.display = 'block';
    } else {
	h.style.display = 'none';
    }
}


function matrix_keyup(e) {
    if (input_active)
	return;

    if (e.ctrlKey===true)
	return;
    
    switch (e.which) {
    case 49:
    case 50:
    case 51:
    case 52:
    case 53:
    case 54:
    case 55:
    case 56:
	matrix_set_marker_size(e.which-48);
	break;

    case 57:
	matrix_set_marker_size(56);
	break;

    case 48:
	if (!e.ctrlKey)
	    matrix_set_marker_size(64);
	break;

    case 86: // 'v'
	matrix_set_show_other(!show_all);
	break;

    case 65: // 'a'
	if (matrix_tool==1) {
	    marker_angle++;
	    if (marker_angle>2)
		marker_angle = 0;
	    set_marker_angle(marker_angle);
	}
	break;

    case 67: // 'C'
	if (e.shiftKey)
	    matrix_clear();
	break;

    case 73: // 'i'
	if (show_info) {
	    show_info = 0;
	    matrix_hide_info();
	} else {
	    show_info = 1;
	    if (inputmarker.style.display!='none') {
		matrix_update_info();
		matrix_show_info();
	    }
	}
	post_call('/wwwdata?matrix_show_info', show_info, false);

	break;

    case 87: // 'w'
	matrix_delete();
	break;

    default:
	//alert(e.which);
    }
    
    if (e.keyCode==16) {
	if (quick_over>0) {
	    quickshot_keys(false, e.ctrlKey || e.metaKey);
	}
    } else if (e.keyCode==17) {
	if (quick_over>0) {
	    quickshot_keys(e.shiftKey, false);
	}
    }
}


function matrix_keydown(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    if (input_active) {
	switch (e.keyCode) {
	case 38: // CU
	    var id = srcElement.labelid;
	    var type = srcElement.labeltype;
	    matrix_labelchanged(e);
	    if (id>0) {
		if (type=='src') {
		    matrix_add_src_input(id-1);
		    matrix_keyscroll(0, -1, e.shiftKey, e.ctrlKey);
		} else {
		    matrix_add_dst_input(id-1);
		    matrix_keyscroll(-1, 0, e.shiftKey, e.ctrlKey);
		}
	    }
	    e.preventDefault();
	    break;
	case 40: // CD 
	    var id = srcElement.labelid;
	    var type = srcElement.labeltype;
	    matrix_labelchanged(e);
	    if (id<1023) {
		if (type=='src') {
		    matrix_add_src_input(id+1);
		    matrix_keyscroll(0, 1, e.shiftKey, e.ctrlKey);
		} else {
		    matrix_add_dst_input(id+1);
		    matrix_keyscroll(1, 0, e.shiftKey, e.ctrlKey);
		}
	    }
	    e.preventDefault();
	    break;
	case 13: // Return
	    matrix_labelchanged(e);
	    break;
	case 27: // ESC
	    matrix_closeinput(e);
	    break;
	    
	default:
	    //alert(e.keyCode);
	}
	
    } else {
	switch (e.keyCode) {
	case 13: // return
	    e.preventDefault(); // prohibit page reload
	    return false;
	    break;
	case 37: // CL
	    matrix_keyscroll(-1, 0, e.shiftKey, e.ctrlKey);
	    break;
	case 38: // CU
	    matrix_keyscroll(0, -1, e.shiftKey, e.ctrlKey);
	    break;
	case 39: // CR
	    matrix_keyscroll(1, 0, e.shiftKey, e.ctrlKey);
	    break;
	case 40: // CD 
	    matrix_keyscroll(0, 1, e.shiftKey, e.ctrlKey);
	    break;
	case 36: // Pos1
	    matrix_scroll(0, 0);
	    break;
	case 35: // End
	    matrix_scroll(matrix_width, matrix_height);
	    break;			
	case 33: // PageDown
	    matrix_quadrant(-1);
	    break;
	case 34: // PageUp
	    matrix_quadrant(1);
	    break;
	}

	if (e.keyCode==16) {
	    if (quick_over>0) {
		quickshot_keys(true, e.ctrlKey || e.metaKey);
	    }
	} else if (e.keyCode==17) {
	    if (quick_over>0) {
		quickshot_keys(e.shiftKey, true);
	    }
	}
    }
}


function matrix_keypress(e) {
    var o;

    if (input_active)
	return;
    
    if (e.ctrlKey===true)
	return;

    switch (e.which) {
    case 13: // return
	//matrix_commit();
	break;
    case 32: // space
	matrixtoolbox_setmode((matrix_mode=='a')?'o':'a', false);
	break;
    case 99: // 'c'
	matrixtoolbox_settool(3);
	break;
    case 100: // 'd'
	matrix_toggle_labels(); 
	break;
    case 104: // 'h'
	matrix_toggle_help();
	break;
    case 106: // 'j'
	matrix_toggle_safety_click();
	break;
    case 108: // 'l'
	matrixtoolbox_settool(2);
	break;
    case 112: // 'p'
	matrix_ports_open();
	break;
    case 113: // 'q'
	quickshot_toggle();
	break;
    case 114: // 'r'
	matrix_recall_trigger(e.shiftKey);
	break;
    case 115: // 's'
	matrix_snapshot(e.shiftKey); 
	break;
    case 120: // 'x'
	matrixtoolbox_settool(1);
	break;
    case 82: // 'R'
	matrix_recall_trigger(e.shiftKey);
	break;
    case 83: // 'S'
	matrix_snapshot(e.shiftKey); 
	break;
    default:
	//alert(e.which);
	//return false;
    }
    
    e.preventDefault();
    return true;
}


function matrix_update_canvas() {
    if (canvas_online) 
	canvas_online.style.display = (show_all || (matrix_mode=='a'))?'block':'none';
    if (canvas_offline) 
	canvas_offline.style.display = (show_all || (matrix_mode=='o'))?'block':'none';
}

/*
  function matrix_octet_removal(ch_in, ch_out, mode) {
  var ctx = (mode=='o')?canvas_offline_ctx:canvas_online_ctx;
  var c = (mode=='o')?config_xpo:config_xpa;

  if (ctx) {
  var x = Math.floor(ch_out/64)*8 + Math.floor((ch_out%64)/8);
  var y = Math.floor(ch_in/64)*8 + Math.floor((ch_in%64)/8);
  var i;

  for (i=x; i<x+8; i++) {
  if ((c[i]>-1) && 
  ((Math.floor(c[i]/64)+Math.floor(c[i]%64)/8)==ch_in))
  return; 
  }
  
  x = Math.floor(ch_out/64)*10 + Math.floor((ch_out%64)/8)+1;
  y = Math.floor(ch_in/64)*10 + Math.floor((ch_in%64)/8)+1;
  ctx.clearRect(x, y, 1, 1);
  }
  }
*/

/*
  function matrix_update_octet(ch_in, ch_out, mode) {
  var ctx = (mode=='o')?canvas_offline_ctx:canvas_online_ctx;

  if (ctx) {
  var x = Math.floor(ch_out/64)*10 + Math.floor((ch_out%64)/8)+1;
  var y = Math.floor(ch_in/64)*10 + Math.floor((ch_in%64)/8)+1;

  ctx.fillRect(x, y, 1, 1);
  }
  }
*/


function matrix_update_octet(ch, mode) {
    var ctx = (mode=='o')?canvas_offline_ctx:canvas_online_ctx;
    var c = (mode=='o')?config_xpo:config_xpa;

    if (ctx) {
	var x = port_out_rev[Math.floor(ch/64)]*10 + Math.floor((ch%64)/8)+1;
	ctx.clearRect(x, 0, 1, 160);
	var base = Math.floor(ch/8)*8;

	for (i=base; i<base+8; i++) {
	    if (c[i]>-1) {
		var p = Math.floor(c[i]/64);
		if (active_input_ports[p]) {
		    var y = port_in_rev[p]*10+Math.floor((c[i]%64)/8)+1;
		    ctx.fillRect(x, port_in_rev[p]*10+Math.floor((c[i]%64)/8)+1, 1, 1);
		}
	    }
	}
    }

}


function matrix_draw_pending(src, dst, mode, action) {
    var o = document.getElementById("xp"+mode+dst+"_p");
    if (!o) {
	o = document.createElement("img");
	matrix.appendChild(o);
    }

    o.src = "/img/matrix_"+mode+action+"_p.png";
    o.id = "xp"+mode+dst+"_p";
    o.className = "xp"+mode;
    o.style.left = (dst*20)+"px";
    o.width = "20px";
    o.height = "20px";
    o.style.top = (src*20)+"px";
}


function matrix_clear_pending(dst, mode) {
    var o = document.getElementById("xp"+mode+dst+"_p");
    if (o) {
	matrix.removeChild(o);
	if (mode=='a')
	    xpa_pending[dst] = -2;
	else if (mode=='o')
	    xpo_pending[dst] = -2;
    }
}


function matrix_draw_xp(src, dst, mode) {
    var o, id;
    var in_ch;
    var inp = Math.floor(src/64);
    var outp = Math.floor(dst/64);

    if (!(active_input_ports[inp] && active_output_ports[outp]))
	return;
    
    // XP icon
    o = document.createElement("img");
    o.className = "xp"+mode;
    o.id = "xp"+mode+dst;
    o.style.left = ((port_out_rev[outp]*64+dst%64)*20)+"px";
    o.width = "20";
    o.height = "20";
    o.style.top = ((port_in_rev[inp]*64+src%64)*20)+"px";
    if (matrix_mode != mode) {
	if (show_all)  // 08.11.2011
	    o.style.opacity = 0.4;
	else
	    o.style.display = 'none';
    }
    matrix.appendChild(o);
    update_xp_marker(dst, mode);
    
    // input channel indicator
    id = "in"+mode+dst+"_c";
    o = document.getElementById(id);
    if (!o) {
	o = document.createElement("img");
	o.src = "/img/matrix_"+mode+".png";
	o.id = id;
	o.className = "xp"+mode+"ic";
	if (mode=='a')
	    o.onclick = function(){ matrix_inputjump(src, 'a'); };
	else
	    o.onclick = function(){ matrix_inputjump(src, 'o'); };
	o.style.top = (src*20+5)+"px";
	o.style.cursor = 'pointer';
	inputs.appendChild(o);
    }

    // output channel indicator
    id = "out"+mode+dst+"_c";
    o = document.getElementById(id);
    if (!o) {
	o = document.createElement("img");
	o.src = "/img/matrix_"+mode+".png";
	o.id = id; 
	o.className = "xp"+mode+"oc";
	if (mode=='a')
	    o.onclick = function(){ matrix_outputjump(dst, 'a'); };
	else
	    o.onclick = function(){ matrix_outputjump(dst, 'o'); };
	o.style.left = (dst*20+5)+"px";
	o.style.cursor = 'pointer';
	outputs.appendChild(o);	
    }
}


function matrix_clear_xp(src, dst, mode) {
    var o = document.getElementById("xp"+mode+dst);

    if (o) {
	matrix.removeChild(o);

	o = document.getElementById("in"+mode+dst+"_c");
	if (o) 
	    inputs.removeChild(o);
	
	o = document.getElementById("out"+mode+dst+"_c");
	if (o)
	    outputs.removeChild(o);

	//matrix_update_octet(dst, mode);
	//matrix_octet_removal(src, dst, mode);
    }
}



function matrix_remove_xp(dst, mode) {
    if (mode=='a') {
	xpa_pending[dst] = -1;
	matrix_draw_pending(config_xpa[dst], dst, 'a', 'd');
	send_cmd("xp?-1,"+dst+",0");

    } else {
	xpo_pending[dst] = -1;
	matrix_draw_pending(config_xpo[dst], dst, 'o', 'd');
	send_cmd("xp?-1,"+dst+",1");
    }
}


function matrix_set_xp(src, dst, mode) {
    if (mode=='a') {
	xpa_pending[dst] = src;
	matrix_draw_pending(src, dst, 'a', 's');
	send_cmd("xp?"+src+","+dst+",0");

    } else {
	xpo_pending[dst] = src;
	matrix_draw_pending(src, dst, 'o', 's');
	send_cmd("xp?"+src+","+dst+",1");
    }
}


function matrix_commit() {
    if (matrix_mode=='o')
	send_cmd("commit_audio_matrix");
}


function matrix_set_mode(newmode) {
    if (newmode==matrix_mode) return;

    switch (newmode) {
    case 'o':
	channel_routing = config_xpo;
	matrix_show_offline();
	break;
    case 'a':
	channel_routing = config_xpa;
	matrix_hide_offline();
	break;
    }  

    matrix_mode = newmode;
    matrix_update_xps();
    matrix_update_canvas();
    matrix_put_marker(marker_x, marker_y);

    if (show_labels==2) {
	for (var i=0; i<1024; i++) {
	    inherit_labels(i);
	}
    }
}


function matrix_set_tool(newtool) {
    var old_marker_size = marker_size;
    if (marker_size > 1) matrix_set_marker_size(1);

    matrix_tool = newtool;

    matrix_set_marker_size(old_marker_size);
}


function matrix_copy() {
    send_cmd("copy_audio_matrix");
}


function matrix_show_offline() {
    var i, o;
    for (i=0; i<1024; i++) {
	if (config_xpo[i]>-1) {
	    o = document.getElementById("xpo"+i);
	    //o.style.display = 'block';
	    if (o) // does not exist if port is hidden!
		o.style.opacity = 1.0;
	}
	if (config_xpa[i]>-1) {
	    o = document.getElementById("xpa"+i);
	    if (o) // does not exist if port is hidden!
		o.style.opacity = 0.4;
	}
    }

    canvas_offline.style.opacity = 1.0;
    canvas_online.style.opacity = 0.4;
}


function matrix_hide_offline() {
    var i;

    for (i=0; i<1024; i++) {
	if (config_xpo[i]>-1) {
	    o = document.getElementById("xpo"+i);
	    if (o)
		o.style.opacity = 0.4;
	}
	if (config_xpa[i]>-1) {
	    o = document.getElementById("xpa"+i);
	    if (o)
		o.style.opacity = 1.0;
	}
    }

    canvas_offline.style.opacity = 0.4;
    canvas_online.style.opacity = 1.0;
}


function matrix_update_xps() {
    var i;

    for (i=0; i<1024; i++) {
	if (config_xpo[i]>-1) {
	    o = document.getElementById("xpo"+i);
	    if (o)
		o.style.display = (show_all || (matrix_mode=='o'))?'block':'none';
	}
	if (config_xpa[i]>-1) {
	    o = document.getElementById("xpa"+i);
	    if (o)
		o.style.display = (show_all || (matrix_mode=='a'))?'block':'none';
	}
    }
}


function update_myname(name) {
    if (name=="") {
	alert("You are not logged in. Please log in on the main page first.");
    }
}


function empty(o) {
    while (o.firstChild)
	o.removeChild(o.firstChild);
}


function cb_matrix_ports(res) {
    var lines = res.split("\n");
    num_input_ports = 0;
    num_output_ports = 0;
    if (lines.length==2) {
	for (var i=0; i<16; i++) {
	    active_input_ports[i] = (lines[0][i]=='x');
	    if (active_input_ports[i]) {
		port_in_rev[i] = num_input_ports;
		port_in[num_input_ports++] = i;
	    } else {
		port_in_rev[i] = -1;
	    }
	    active_output_ports[i] = (lines[1][i]=='x');
	    if (active_output_ports[i]) {
		port_out_rev[i] = num_output_ports;
		port_out[num_output_ports++] = i;
	    } else {
		port_out_rev[i] = -1;
	    }
	}
	i = num_input_ports;
	while (i<16)
	    port_in[i++] = -1;
	i = num_output_ports;
	while (i<16)
	    port_out[i++] = -1;
	
    } else {
	// using defaults
	for (var i=0; i<16; i++) {
	    active_input_ports[i] = true;
	    port_in_rev[num_input_ports] = i;
	    port_in[num_input_ports++] = i;						
	    active_output_ports[i] = true;
	    port_out_rev[num_output_ports] = i;
	    port_out[num_output_ports++] = i;
	}
    }

    empty(inputs);
    empty(outputs);
    empty(matrix);
    matrixmarker = document.createElement("div");
    matrixmarker.id = 'matrixmarker';
    matrix.appendChild(matrixmarker);
    matrix.style.width = (num_output_ports*1280)+"px";
    matrix.style.height = (num_input_ports*1280)+"px";
    matrix_width = num_output_ports*1280;
    matrix_height = num_input_ports*1280;
    
    var bgcolor;
    
    for (i=0; i<num_input_ports; i++) {
	if (i%2==0) {
	    bgcolor = "333";
	} else {
	    bgcolor = "666";
	}
	matrix_addinputport(port_in[i], i*1280, labels_input_ports[port_in[i]], bgcolor);

	for (c=0; c<64; c++) {
	    if (c%16<8) {
		bgcolor = "B1B1B1";
	    } else {
		bgcolor = "BBBBBB";
	    }				
	    matrix_addinput(port_in[i]*64+c, i*64*20+c*20);
	}
    }

    matrixmap_redraw();

    for (i=0; i<num_output_ports; i++) {
	if (i%2==0) {
	    bgcolor = "333";
	} else {
	    bgcolor = "666";
	}
	matrix_addoutputport(port_out[i], i*1280, labels_output_ports[port_out[i]], bgcolor);

	for (c=0; c<64; c++) {
	    if (c%16<8) {
		bgcolor = "B1B1B1";
	    } else {
		bgcolor = "BBBBBB";
	    }				
	    matrix_addoutput(port_out[i]*64+c, i*64*20+c*20);

	    esc_xp(0, config_xpa[port_out[i]*64+c], port_out[i]*64+c);
	    esc_xp(1, config_xpo[port_out[i]*64+c], port_out[i]*64+c);
	}
    }

    redraw_labels();


}



function matrix_ports_open() {
    var o = document.getElementById('matrix_ports');
    if (o) {
	o.style.display = 'block';
    }
    for (var i=0; i<16; i++) {
	var o = document.getElementById('matrix_input_port_'+(i+1));
	o.checked = active_input_ports[i];
	//o.disabled = !permitted_input_ports[i];
	if (!permitted_input_ports[i])
	    o.style.opacity = 0.6;
	
	var o = document.getElementById('matrix_output_port_'+(i+1));
	o.checked = active_output_ports[i];
	//o.disabled = !permitted_output_ports[i];
	if (!permitted_output_ports[i])
	    o.style.opacity = 0.6;
    }		
}

function cb_matrix_ports_applied() {
    var o = document.getElementById('matrix_ports');
    if (o) {
	o.style.display = 'none';
    }
    var instr = "";
    var outstr = "";
    
    for (var i=0; i<16; i++) {
	var o = document.getElementById('matrix_input_port_'+(i+1));
	instr += o.checked?"x":" ";
	
	var o = document.getElementById('matrix_output_port_'+(i+1));
	outstr += o.checked?"x":" ";
    }
    post_call('/wwwdata?matrix_ports', instr+"\n"+outstr, false);
    ajax_call('/wwwdata?matrix_ports', cb_matrix_ports);
}


function cb_quickshot_list(res) {
    var shots = res.split(';');
    var o;

    for (i=1; i<=9; i++) {
	o = document.getElementById("q"+i);
	o.className = 'empty';
	o.empty = 1;
    }

    for (var i=0; i<shots.length; i++) {
	if (shots[i]!="") {
	    var o = document.getElementById("q"+shots[i]);
	    o.className = 'full';
	}
	o.empty = 0;
    }
}


function quickshot_click(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }
    
    if ((e.ctrlKey && e.shiftKey) || (e.shiftKey && e.metaKey)) { // Command on OS X
	send_cmd("quickshot_delete?"+srcElement.shotid);
	ajax_call('/cmd/quickshot_list', cb_quickshot_list);
	notifications_show("Quickshot "+srcElement.shotid+" deleted");
    }	else if (e.shiftKey) {
	send_cmd("quickshot_save?"+srcElement.shotid+","+((matrix_mode=='a')?0:1));
	ajax_call('/cmd/quickshot_list', cb_quickshot_list);
	notifications_show("Quickshot "+srcElement.shotid+" stored");
    } else if (srcElement.empty==0) {
	send_cmd("quickshot_load?"+srcElement.shotid+","+((matrix_mode=='a')?0:1));
	notifications_show("Quickshot "+srcElement.shotid+" loaded");
    }
}


function quickshot_update(elem, shift, ctrl, over) {
    if (elem.empty) {
	if (shift && !ctrl && over) {
	    elem.style.backgroundImage = "url(/img/quickshot_store.png)";
	    elem.title = "Store in slot "+elem.shotid;
	    elem.style.cursor = 'pointer';
	} else {
	    elem.style.backgroundImage = "url(/img/quickshot_empty.png)";
	    elem.title = "";
	    elem.style.cursor = 'default';
	}
	
    } else {
	if (over) {
	    if (shift && ctrl) {
		elem.style.backgroundImage = "url(/img/quickshot_delete.png)";
		elem.title = "Delete quickshot "+elem.shotid;
		elem.style.cursor = 'pointer';
	    } else if (shift) {
		elem.style.backgroundImage = "url(/img/quickshot_full_store.png)";
		elem.title = "Overwrite quickshot "+elem.shotid;
		elem.style.cursor = 'pointer';
	    } else {
		elem.style.backgroundImage = "url(/img/quickshot_load.png)";
		elem.title = "Load quickshot "+elem.shotid;
		elem.style.cursor = 'pointer';
	    }
	} else {
	    elem.style.backgroundImage = "url(/img/quickshot_full.png)";
	    elem.title = "";
	    elem.style.cursor = 'pointer';
	}
    }
}


function quickshot_mouseover(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }
    
    quick_over = srcElement.shotid;

    quickshot_update(srcElement, e.shiftKey, e.ctrlKey || e.metaKey, true);
}


function quickshot_mouseout(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    quick_over = 0;

    quickshot_update(srcElement, e.shiftKey, e.ctrlKey || e.metaKey, false);
}


function quickshot_keys(shift, ctrl) {
    var o = document.getElementById("q"+quick_over);

    quickshot_update(o, shift, ctrl, true);
}




function cb_show_quickshot(res) {
    if (res=="1") {
	var q = document.getElementById('quickshot');
	q.style.display = "block";
    }
}


function quickshot_init() {
    var i, o;

    for (i=1; i<=9; i++) {
	o = document.getElementById("q"+i);
	o.shotid = i;
	o.onclick = quickshot_click;
	o.onmouseover = quickshot_mouseover;
	o.onmouseout = quickshot_mouseout;
    }


    ajax_call('/cmd/quickshot_list', cb_quickshot_list);
}


function quickshot_toggle() {
    var q = document.getElementById('quickshot');
    q.style.display = (q.style.display=='none')?"block":"none";
    post_call('/wwwdata?matrix_show_quickshot', (q.style.display=='block')?"1":"0", false);
}


function toggle_ports(row) {
    var allon=true;

    for (var i=0; i<16; i++) {
	var o = document.getElementById("matrix_"+row+"_port_"+(i+1));
	if (o) {
	    if (!o.checked) {
		allon = false;
	    }
	}
    }

    for (var i=0; i<16; i++) {
	var o = document.getElementById("matrix_"+row+"_port_"+(i+1));
	if (o) {
	    o.checked = !allon;
	}
    }
    
}


function matrix_init() {
    var i, o;

    ajax_call('/data/myname', update_myname);

    matrixmap = document.getElementById("matrixmap");

    matrix = document.getElementById("matrix");
    matrixmarker = document.getElementById("matrixmarker");
    matrixviewport = document.getElementById("matrixviewport");
    mapmoepel = document.getElementById("mapmoepel");
    inputs = document.getElementById("inputs");
    inputmarker = document.getElementById("inputmarker");
    outputs = document.getElementById("outputs");
    outputmarker = document.getElementById("outputmarker");

    matrixmap.addEventListener("mousedown", matrixmap_mousedown, true);
    matrixmap.addEventListener("mouseup", matrixmap_mouseup, true);
    matrixmap.addEventListener("mousemove", matrixmap_mousemove, true);
    matrixmap.addEventListener("mouseout", matrixmap_mouseout, true);

    matrixviewport.addEventListener("mousedown", matrix_mousedown, true);
    matrixviewport.addEventListener("mouseup", matrix_mouseup, true);
    matrixviewport.addEventListener("mousemove", matrix_mousemove, true);
    matrixviewport.addEventListener("mouseout", matrix_mouseout, true);
    matrixviewport.addEventListener("mouseover", matrix_mouseenter, true);
    
    matrixviewport.addEventListener("DOMMouseScroll", matrix_mousewheel, true);
    matrixviewport.addEventListener('mousewheel', matrix_mousewheel, false);

    window.addEventListener("resize", matrixmap_moepelresize, true);
    matrixmap_moepelresize();

    //channel_routing = xpo;

    canvas_online = document.getElementById('canvas_online');
    if (canvas_online) {
	canvas_online_ctx = canvas_online.getContext('2d');
	canvas_online_ctx.fillStyle = "rgb(121, 192, 64)";
    }
    canvas_offline = document.getElementById('canvas_offline');
    if (canvas_offline) {
	canvas_offline_ctx = canvas_offline.getContext('2d');
	canvas_offline_ctx.fillStyle = "rgb(210, 150, 64)";
    }

    /*
      var bgcolor;
      for (i=0; i<1024; i++) {
      if (i%16<8) {
      bgcolor = "B1B1B1";
      } else {
      bgcolor = "BBBBBB";
      }
      matrix_addinput(i, i*20);
      matrix_addoutput(i, i*20);
      }

      for (i=0; i<16; i++) {
      if (i%2==0) {
      bgcolor = "333";
      } else {
      bgcolor = "666";
      }
      matrix_addinputport(i, i*1280, labels_input_ports[i], bgcolor, '000');
      matrix_addoutputport(i, i*1280, labels_output_ports[i], bgcolor, '000');

      }
    */
    
    matrix_scroll(0, 0);
    matrix_put_marker(marker_x, marker_y);

    document.addEventListener('keydown', matrix_keydown, false);  
    document.addEventListener('keypress', matrix_keypress, true);  
    document.addEventListener('keyup', matrix_keyup, true);  

    ajax_call('/wwwdata?matrix_ports', cb_matrix_ports);
    ajax_call('/wwwdata?matrix_show_info', cb_show_info);
    ajax_call('/wwwdata?matrix_show_other', cb_show_other);
    ajax_call('/wwwdata?show_labels', cb_show_labels);
    ajax_call('/wwwdata?matrix_show_quickshot', cb_show_quickshot);
    ajax_call('/wwwdata?safety_click', cb_safety_click);
}

matrix_init();
ws_init(events_connected, events_disconnected);
notifications_init();
