
var freq_min = 32000;
var bg_low = 'cccc00';


function send_midi() {
  var str = document.getElementById('midi_data').value;
  
  if (str.match(/^(\s*[a-fA-F0-9]{2}\s*)+$/))
    send_cmd("midi?"+encodeURI(str));
}


function check_midi() {
  var ta = document.getElementById('midi_data');
  
  if (ta.value.match(/^(\s*[a-fA-F0-9]{2}\s*)+$/)) {
    ta.style.backgroundColor="white"; 
    //document.getElementById("btn_send").disabled=false; 
    var matches = ta.value.match(/(\s*[a-fA-F0-9]{2}\s*)/g);
    document.getElementById("midi_rest").innerHTML = matches.length+"/200 MIDI bytes";

  } else { 
    ta.style.backgroundColor="red"; 
    //document.getElementById("btn_send").disabled=true; 
  }
}

function update_routing() {
  var sel = document.getElementById('midi_in');
  var sv = get_selectactive(sel);
  send_cmd("midixp?17,"+sv);

  for (var i=0; i<17; i++) {
    var v = document.getElementById('midi_out_'+i).checked;
    send_cmd("midixp?"+i+","+(v?17:-1));
  }
}


//
// wwwdata callbacks
//


function set_display_labels(val) {
  display_labels = val;
  document.getElementById('cb_show_labels').checked = display_labels;
  if (channel_info.style.display=='block')
    update_channel_info(last_port, last_ch);
  post_call('/wwwdata?display_labels', val, false);
}


//
// Config callbacks
//


function esc_midixp(p_in, p_out) {
  if (p_out<17)
    document.getElementById('midi_out_'+p_out).checked = (p_in==17);

  if (p_out==17) {
    set_selectactive(document.getElementById("midi_in"), p_in);
  }
}


function ess_freq(port, freq) {
	if (port>15)
		return;

	var o = document.getElementById("label_out_"+port);
	if (freq<freq_min) {
		o.style.color = "#"+bg_low;
		o.title = "Clock too low for MIDI transmission";
    document.getElementById("midi_in_"+port).style.color = "#"+bg_low;
	} else {
		o.style.color = '#000000';
		o.title = "";
    document.getElementById("midi_in_"+port).style.color = "#000000";
	}
}


//
// MIDI callbacks
//


function midi_cmd(str) {
  if (str[0]=='F')
    add_log(str, 'rt');
  else
    add_log(str, 'cmd');
}


function midi_sysex(str) {
  add_log(str, 'sysex');
}

function midi_illegal(str) {
  add_log(str, 'illegal');
}

//


function permissions_updated() {
  for (var i=0; i<17; i++) {
    document.getElementById("midi_out_"+i).disabled = (parent.frames[0].permissions.midi_output[i]==0);
    document.getElementById("label_out_"+i).className = (parent.frames[0].permissions.midi_output[i])?"":"disabled";
  }
  for (var i=0; i<18; i++)
    document.getElementById("midi_in_"+i).disabled = (parent.frames[0].permissions.midi_input[i]==0);
}


//
//
//

var midi_connected;

function midi_callback(event) {
  if (midi_connected==0) {
    midi_connected = 1;
    document.getElementById('log').style.opacity = 1;
  }

  var rpc = JSON.parse(event.data);
  
  if (window[rpc[0]])
    window[rpc[0]](rpc[1]);
}

function midi_disconnected() {
  if (midi_socket.readyState!=midi_socket.CLOSED)
    midi_socket.close();

  midi_connected = 0;
  document.getElementById('log').style.opacity = 0.5;

  window.setTimeout("midi_connect()", 1000);
}

function midi_connect() {
  midi_socket = new WebSocket('ws://'+window.location.host+'/ws/midi');
  midi_socket.onmessage = midi_callback;
  midi_socket.onclose = midi_disconnected;   
}

function midi_init() {
  midi_connect();

  for (var i=0; i<18; i++)
    esc_midixp(parent.frames[0].config_midi[i], i);

	for (var i=0; i<16; i++)
		ess_freq(i, parent.frames[0].status_port_freq[i]);

  permissions_updated();
}
